from typing import Optional, List

from secoda_common.models.table import Badge

import attr
from marshmallow_annotations.ext.attrs import AttrsSchema


@attr.s(auto_attribs=True, kw_only=True)
class LineageItem:
    key: str  # down/upstream table/col/task key
    name: Optional[str] = ""  # name for dashboards
    level: int  # upstream/downstream distance from current resource
    type: str  # indicate resouce type
    native_type: Optional[str] = None
    group_name: Optional[str] = ""  # group name for dashboards
    description: Optional[str]
    badges: Optional[List[Badge]] = None
    usage: Optional[int] = None  # statistic to sort lineage items by
    upstream_entities: Optional[List[str]] = []  # list of upstream table/col/task keys
    downstream_entities: Optional[
        List[str]
    ] = []  # list of downstream table/col/task keys


class LineageItemSchema(AttrsSchema):
    class Meta:
        target = LineageItem
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Lineage:
    key: str  # current table/col/task key
    direction: str  # upstream/downstream/both
    depth: int  # how many levels up/down 0 == all
    upstream_entities: List[LineageItem]  # list of upstream entities
    downstream_entities: List[LineageItem]  # list of downstream entities


class LineageSchema(AttrsSchema):
    class Meta:
        target = Lineage
        register_as_scheme = True
